﻿// AStar pathfinding Aaron Steed 2007
// Based on the Carden / Steed algorithm
class AStar {
	var wide:Number;
	var high:Number;
	var grid:Array;
	var open:Array;
	var closed:Array;
	function AStar(w:Number, h:Number) {
		this.wide = w;
		this.high = h;
		open = new Array();
		closed = new Array();
		makeMap();
	}
	// Returns an Array of Nodes defining the shortest distance between x0,y0 & x1,y1
	// Note that the A* map is a scale version of your game map - it locates the grid
	// node in a linear array.
	function getPath(x0:Number, y0:Number, x1:Number, y1:Number):Array {
		var start = grid[x0 + y0 * wide];
		var finish = grid[x1 + y1 * wide];
		if (start == finish) {
			return new Array();
		}
		open = new Array();
		closed = new Array();
		for (var i = 0; i < grid.length; i++) {
			grid[i].reset();
		}
		open.push(start);
		while (open.length > 0) {
			var lowest = Number.MAX_VALUE;
			var c = -1;
			for (var i = 0; i < open.length; i++) {
				if (open[i].f < lowest) {
					lowest = open[i].f;
					c = i;
				}
			}
			var current:Node = open.splice(c, 1)[0];
			closed.push(current);
			if (current == finish) {
				break;
			}
			for (var i = 0; i < current.links.length; i++) {
				var adjacent = current.links[i];
				if (adjacent.walkable && !arrayContains(closed, adjacent)) {
					if (!arrayContains(open, adjacent)) {
						open.push(adjacent);
						adjacent.parent = current;
						adjacent.setF(finish);
					} else {
						if (adjacent.g > current.g + current.dist(adjacent)) {
							adjacent.parent = current;
							adjacent.setF(finish);
						}
					}
				}
			}
		}
		var path = new Array();
		var pathNode:Node = finish;
		while (pathNode != null) {
			path.push(pathNode);
			pathNode = pathNode.parent;
		}
		return path;
	}
	// Generate a new empty map
	function makeMap():Void {
		grid = new Array();
		for (var y = 0; y < high; y++) {
			for (var x = 0; x < wide; x++) {
				grid[x + y * wide] = new Node(x, y, x + y * wide);
			}
		}
		for (var i = 0; i < grid.length; i++) {
			for (var x = -1; x < 2; x++) {
				for (var y = -1; y < 2; y++) {
					var lx = grid[i].x + x;
					var ly = grid[i].y + y;
					if (!(x == 0 && y == 0) && lx > -1 && lx < wide && ly > -1 && ly < high) {
						grid[i].addLink(grid[lx + ly * wide]);
					}
				}
			}
		}
	}
	// Disconnect all incoming links on !walkable Nodes
	function disconnectUnwalkables():Void {
		for (var i = 0; i < grid.length; i++) {
			if (!grid[i].walkable) {
				grid[i].disconnect();
			}
		}
	}
	// Create a square path around !walkable Nodes
	function radialDisconnectUnwalkables():Void {
		for (var i = 0; i < grid.length; i++) {
			if (!grid[i].walkable) {
				grid[i].radialDisconnect();
			}
		}
	}
	// Returns true if myArray contains myObject
	function arrayContains(myArray:Array, myObject:Object):Boolean {
		for (var i = 0; i < myArray.length; i++) {
			if (myArray[i] == myObject) {
				return true;
			}
		}
		return false;
	}
}
